trap fo_close INT QUIT TERM EXIT

readonly FO_SCRIPT=$(basename "${0}")
readonly RED="\033[1;41m"
readonly NORM="\033[0m"

auxilium_add nolock  -t opt -m "Don\'t create lock file"

another_instance() {
  logger -t "udev" "===> There is another instance running, exiting"
  exit 1
}

fo_repeat_char(){
  # repeat a char X 
	# param: char to repeat
	# param: nb of repeat
	local c="$1"
	local nb=$(seq 1 ${2})
	printf -- "${c}%.0s" ${nb}
}

fo_center_align(){
	# Center align a text
	# Param: Text to align
	# Param: len of final string
	# Param: If is disk to locate
	local text=$1
	local lenTotal=$2
	local locate=$3
	local lenText=${#text}
	local nb_spaces=$(($lenTotal - $lenText))
	local space=' '
	local color=${NORM}
	if [ "$locate" == "True" ];then
		color=$RED
		[ -n "$MAXILIUM_ARG_redmine" ] && space='='
	fi
	local first_spaces=$(fo_repeat_char "${space}" $(($nb_spaces / 2)))
	local end_spaces=$(fo_repeat_char "${space}" $(($nb_spaces - ($nb_spaces / 2))))
	local ret=$(printf -- "%s%s%s" "" "${first_spaces}" "$text" "${end_spaces}")
	printf -- "${color}%${lenTotal}s${NORM}" "$ret"
}

fo_get_dev_letter(){
	# Print disk letter (a to z) by is index 
	# Param: Disk Index (beginning at 0)
	local num=$(( 97 + ${1} ))
	printf \\$(printf '%03o' ${num})
}

fo_get_dev_ascii(){
	# Print disk ascii code of last letter of device name
	# Param: letter
	local name_id=$(printf "%d\n" \'${1:(-1)})
	echo $(( ${name_id} - 97))
}

fo_lock() {
	local LOCKDIR=/var/run/lock
	readonly FO_LOCKFILE=${LOCKDIR}/$(basename ${0})
	if [ -z "$MAXILIUM_ARG_nolock" ] ; then
		logger -t "udev" "===> Claming lock for $0 in ${FO_LOCKFILE}"
		if [ -f "$FO_LOCKFILE" ]; then
			kill -0 "$(cat $FO_LOCKFILE)" && another_instance
		fi
		echo $$ > "$FO_LOCKFILE"
	fi
}

fo_use_cache_file() {
	# Test if a cache file is valid, and delete it if invalid
	# Param: Cache File Suffix
	local usecache=false
	# read a cache file, if exist and recent
	if [ -f "$1" ]; then
		local lastentry=$(stat -c %Y "$1")
		local lastvalid=$(date -d "now -5 min" +%s)
		if [ $lastentry -gt $lastvalid ]; then
			usecache=true
		else
			rm "$1"
		fi
	elif [ -e "$1" ]; then
		rm "$1"
	fi
	if ! $usecache; then
		logger -t "udev" "[Rebuilding cache file]"
		# We should rebuild the cache: exit 1
		return 1
	else
		logger -t "udev" "[Using cache file]"
		# We can re-use the cache: exit 0
		return 0
	fi
}

fo_load_lshw(){
	fo_use_cache_file "${FO_CACHE_FILE_LSHW}" || lshw -c disk -json | jq '.[]|select(.class=="disk")|{id, businfo, vendor, serial, product, version, size, physid, logicalname, description}' > "${FO_CACHE_FILE_LSHW}"
}

fo_close() {
  auxilium_close
	rm $FO_LOCKFILE 2>/dev/null
}

if [ -x /usr/bin/storcli64 ] ; then
	readonly STORCLI=/usr/bin/storcli64
elif [ -x /usr/bin/storcli_x64 ] ; then
	readonly STORCLI=/usr/bin/storcli_x64
elif [ -x /usr/bin/storcli ] ; then
	readonly STORCLI=/usr/bin/storcli
fi
readonly FO_ETC_PLUGINS_PATH="/etc/ferreus-orbit/plugins"
if [ -d "/usr/share/ferreus-orbit/plugins" ]; then
	readonly FO_PLUGINS_PATH="/usr/share/ferreus-orbit/plugins"
else
	readonly FO_PLUGINS_PATH="../share/plugins"
fi

readonly FO_CHASSIS_MANUFACTURER=$(dmidecode -s system-manufacturer)
readonly FO_CHASSIS_MODEL=$(dmidecode -s system-product-name)

if [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
elif [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
  . "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
else
  . "${FO_PLUGINS_PATH}/generic"
fi

readonly LSHW=/usr/bin/lshw
readonly FO_CACHE_FILE_LSHW=/tmp/${FO_SCRIPT}_cache_lshw
